import xml.sax

from ggps.base_handler import BaseHandler
from ggps.trackpoint import Trackpoint


class GpxHandler(BaseHandler):

    tkpt_path = "gpx|trk|trkseg|trkpt"  # ggps_src
    tkpt_path_len = len(tkpt_path)

    def parse(self, filename):
        xml.sax.parse(open(filename), self)
        return self

    def __init__(self):
        BaseHandler.__init__(self)

    def startElement(self, tag_name, attrs):
        self.heirarchy.append(tag_name)
        self.reset_curr_text()
        path = self.curr_path()

        if path == self.tkpt_path:
            self.curr_tkpt = Trackpoint()
            lat, lon = attrs["lat"], attrs["lon"]
            self.curr_tkpt.set("latitudedegrees", lat)
            self.curr_tkpt.set("longitudedegrees", lon)
            self.trackpoints.append(self.curr_tkpt)
            return

    def endElement(self, tag_name):
        path = self.curr_path()

        if self.tkpt_path in path:
            if len(path) > self.tkpt_path_len:
                retain = True
                if tag_name == "ele":
                    retain = False
                elif tag_name == "extensions":
                    retain = False
                elif tag_name == "gpxtpx:TrackPointExtension":
                    retain = False
                elif tag_name == "gpxtpx:hr":
                    tag_name = "heartratebpm"
                elif tag_name == "ns3:hr":
                    tag_name = "heartratebpm"

                elif tag_name == "gpxtpx:cad":
                    tag_name = "cadence"
                elif tag_name == "ns3:cad":
                    tag_name = "cadence"

                if retain:
                    self.curr_tkpt.set(tag_name, self.curr_text)

        self.heirarchy.pop()
        self.reset_curr_text()

    def endDocument(self):
        self.end_reached = True
        for idx, t in enumerate(self.trackpoints):
            if idx == 0:
                self.set_first_trackpoint(t)

            t.set("seq", "{0}".format(idx + 1))
            self.calculate_elapsed_time(t)
            self.cadence_x2(t)



import json

import ggps


def test_str():
    handler = ggps.PathHandler()
    actual = str(handler)
    expected = "{}"
    assert actual == expected

    filename = "data/twin_cities_marathon.gpx"
    handler = ggps.PathHandler()
    handler.parse(filename)
    obj = json.loads(str(handler))
    cnt = obj["gpx|trk|trkseg|trkpt@lat"]
    assert cnt == 2256


def test_counts():
    filename = "data/twin_cities_marathon.gpx"
    handler = ggps.PathHandler()
    handler.parse(filename)
    counter = handler.path_counter
    cnt = counter["gpx|trk|trkseg|trkpt@lat"]
    assert cnt == 2256
    cnt = counter["gpx|metadata|time"]
    assert cnt == 1


def test_base_parse_hhmmss():
    filename = "data/twin_cities_marathon.gpx"
    handler = ggps.PathHandler()
    handler.parse(filename)
    hhmmss = handler.parse_hhmmss("")
    assert hhmmss == ""
    hhmmss = handler.parse_hhmmss("xxx")
    assert hhmmss == ""



import json
import xml.sax

from collections import defaultdict

from ggps.base_handler import BaseHandler


class PathHandler(BaseHandler):

    def parse(self, filename):
        xml.sax.parse(open(filename), self)
        return self

    def __init__(self):
        BaseHandler.__init__(self)
        self.path_counter = defaultdict(int)

    def startElement(self, name, attrs):
        self.heirarchy.append(name)
        path = self.curr_path()
        self.path_counter[path] += 1

        for aname in attrs.getNames():
            self.path_counter[path + "@" + aname] += 1

    def endElement(self, name):
        self.heirarchy.pop()

    def curr_path(self):
        return "|".join(self.heirarchy)

    def __str__(self):
        return json.dumps(self.path_counter, sort_keys=True, indent=2)



import ggps


def expected_first_trackpoint():
    return {
        "elapsedtime": "00:00:00",
        "heartratebpm": "85",
        "latitudedegrees": "44.97431952506304",
        "longitudedegrees": "-93.26310088858008",
        "seq": "1",
        "time": "2014-10-05T13:07:53.000Z",
        "type": "Trackpoint",
    }


def expected_middle_trackpoint():
    return {
        "elapsedtime": "03:13:19",
        "heartratebpm": "140",
        "latitudedegrees": "44.959017438814044",
        "longitudedegrees": "-93.21290854364634",
        "seq": "1747",
        "time": "2014-10-05T16:21:12.000Z",
        "type": "Trackpoint",
    }


def expected_last_trackpoint():
    return {
        "elapsedtime": "04:14:24",
        "heartratebpm": "161",
        "latitudedegrees": "44.95180849917233",
        "longitudedegrees": "-93.10493202880025",
        "seq": "2256",
        "time": "2014-10-05T17:22:17.000Z",
        "type": "Trackpoint",
    }


def test_twin_cities_marathon_gpx_file():
    filename = "data/twin_cities_marathon.gpx"
    handler = ggps.GpxHandler()
    handler.parse(filename)

    tkpts = handler.trackpoints
    expected_attr_count = 7

    # check the number of trackpoints
    actual = len(tkpts)
    expected = 2256
    assert actual == expected

    # check the first trackpoint
    expected_tkpt = expected_first_trackpoint()
    actual_tkpt = handler.trackpoints[0]
    assert len(actual_tkpt.values) == expected_attr_count
    for key in expected_tkpt.keys():
        expected, actual = expected_tkpt[key], actual_tkpt.values[key]
        assert expected == actual

    # check a trackpoint at ~mile 20
    expected_tkpt = expected_middle_trackpoint()
    actual_tkpt = handler.trackpoints[1746]
    assert len(actual_tkpt.values) == expected_attr_count
    for key in expected_tkpt.keys():
        expected, actual = expected_tkpt[key], actual_tkpt.values[key]
        assert expected == actual

    # check the last trackpoint
    expected_tkpt = expected_last_trackpoint()
    actual_tkpt = handler.trackpoints[-1]
    assert len(actual_tkpt.values) == expected_attr_count
    for key in expected_tkpt.keys():
        expected, actual = expected_tkpt[key], actual_tkpt.values[key]
        assert expected == actual

    # check seconds_to_midnight
    secs = int(handler.first_time_secs_to_midnight)
    assert secs > 0
    assert secs < 86400



import json

import ggps


def test_str():
    t = ggps.Trackpoint()
    assert str(t) == "<Trackpoint values count:1>"


def test_repr():
    t = ggps.Trackpoint()
    t.set("lang", "python")
    t.set(None, "python")
    j = repr(t)
    obj = json.loads(j)
    assert obj["type"] == "Trackpoint"
    assert obj["lang"] == "python"


def test_get():
    t = ggps.Trackpoint()
    t.set("lang", "python")
    assert t.get("lang") == "python"
    assert t.get("xxx") == ""
    assert t.get("zzz", "zero") == "zero"



import json

import ggps

# Use:
# python sample-program.py > tmp/sample-program.txt


if __name__ == "__main__":

    print("ggps version {}".format(ggps.VERSION))

    if True:
        infile = "data/twin_cities_marathon.tcx"
        print("")
        print("processing file {} with ggps.TcxHandler() ...".format(infile))
        handler = ggps.TcxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        for t in trackpoints:
            print(repr(t))
        count = len(trackpoints)
        print("{} trackpoints loaded from file {}".format(count, infile))

    if True:
        infile = "data/twin_cities_marathon.gpx"
        print("==========")
        print("processing file {} with ggps.GpxHandler() ...".format(infile))
        handler = ggps.GpxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        for t in trackpoints:
            print(repr(t))
        count = len(trackpoints)
        print("{} trackpoints loaded from file {}".format(count, infile))

    if True:
        infile = "data/activity_4564516081.tcx"
        print("==========")
        print("processing file {} with ggps.TcxHandler() ...".format(infile))
        handler = ggps.TcxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        for t in trackpoints:
            print(repr(t))
        count = len(trackpoints)
        print("{} trackpoints loaded from file {}".format(count, infile))

    if True:
        infile = "data/twin_cities_marathon.tcx"
        print("==========")
        print("processing file {} with ggps.PathHandler() ...".format(infile))
        handler = ggps.PathHandler()
        handler.parse(infile)
        print(str(handler))
        obj = json.loads(str(handler))

    if True:
        infile = "data/activity_4564516081.gpx"
        print("==========")
        print("processing file {} with ggps.GpxHandler() ...".format(infile))
        handler = ggps.GpxHandler()
        handler.parse(infile)
        trackpoints = handler.trackpoints
        for t in trackpoints:
            print(repr(t))
        count = len(trackpoints)
        print("{} trackpoints loaded from file {}".format(count, infile))



"""
ggps library
"""

__author__ = "Chris Joakim <christopher.joakim@gmail.com>"
__version__ = "0.4.1"

AUTHOR = __author__
VERSION = __version__

from ggps.trackpoint import Trackpoint
from ggps.base_handler import BaseHandler
from ggps.gpx_handler import GpxHandler
from ggps.path_handler import PathHandler
from ggps.tcx_handler import TcxHandler
